--[[
********************************************************************************
Twitter Plugin for Corona SDK Demo Project
Copyright (c) 2016 Jason Schroeder
http://www.jasonschroeder.com
http://www.twitter.com/schroederapps
********************************************************************************
]]--

--------------------------------------------------------------------------------
-- REQUIRE & INITIALIZE TWITTER PLUGIN
--------------------------------------------------------------------------------
local twitter = require "plugin.twitter"
twitter.showAlerts = true
twitter.init("API_KEY", "API_SECRET")

--------------------------------------------------------------------------------
-- LOCAL POSITIONING VARIABLES
--------------------------------------------------------------------------------
local centerX = display.contentCenterX
local centerY = display.contentCenterY
local screenTop = math.floor(display.screenOriginY)
local screenLeft = math.floor(display.screenOriginX)
local screenBottom = display.contentHeight - screenTop
local screenRight = display.contentWidth - screenLeft
local screenWidth = screenRight - screenLeft
local screenHeight = screenBottom - screenTop

--------------------------------------------------------------------------------
-- FORWARD DECLARE LOCAL VARIABLES
--------------------------------------------------------------------------------
local widget = require("widget")
local json = require("json")

-- DISPLAY GROUPS:
local backGroup, frontGroup, waves

-- DISPLAY OBJECTS:
local bg, failwhale, wave1, wave2, title, subtitle, userText, userImage, loginText, getFollowersButton, tweetButton, getUserButton, searchButton, getFriendsButton, followButton, getProfileImageButton, flexibleRequestButton

-- FUNCTIONS:
local tableToString, waveMotion, whaleMotion, showResults, login, logout, showButtons, hideButtons, getFollowers, tweet, getUser, search, getFriends, follow, getProfileImage, flexibleRequest

-- MISC:
local loggedIn = false
local friends = {}

--------------------------------------------------------------------------------
-- CONVERT TABLE TO STRING
--------------------------------------------------------------------------------
function tableToString ( t )
	local output = {}
    local print_r_cache={}
    local function sub_print_r(t,indent)
    	local function print(string)
    		output[#output + 1] = string
    	end
        if (print_r_cache[tostring(t)]) then
            print(indent.."*"..tostring(t))
        else
            print_r_cache[tostring(t)]=true
            if (type(t)=="table") then
                for pos,val in pairs(t) do
                    if (type(val)=="table") then
                        print(indent.."["..pos.."] => "..tostring(t).." {")
                        sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                        print(indent..string.rep(" ",string.len(pos)+6).."}")
                    elseif (type(val)=="string") then
                        print(indent.."["..pos..'] => "'..val..'"')
                    else
                        print(indent.."["..pos.."] => "..tostring(val))
                    end
                end
            else
                print(indent..tostring(t))
            end
        end
    end

	sub_print_r(t,"  ")

    return output
end

--------------------------------------------------------------------------------
-- DISPLAY RESULTS IN SCROLLVIEW
--------------------------------------------------------------------------------
function showResults(results, isError, event, skipHeader)
	hideButtons()

	transition.cancel(failwhale)
	transition.to(failwhale, {y = screenBottom + centerY, time = 800, transition = easing.inOutBack})

	local resultType = type(results)

	if resultType == "table" then
		results = tableToString(results)
	else
		local jsonDecoded = json.decode(results)
		if jsonDecoded ~= nil then
			results = tableToString(results)
		else
			local text = results
			results = {text}
		end
	end

	if isError then
		results = tableToString(event)
	end

	if not skipHeader then
		if not isError then
			results[1] = "************************************************************\nYOUR REQUEST RETURNED THE FOLLOWING " .. string.upper(resultType) .. ":\n************************************************************\n\n"..results[1]
		else
			results[1] = "************************************************************\nTWITTER PLUGIN ERROR. HERE ARE DETAILS: " .. "\n************************************************************\n\n"..results[1]
		end
	end

	local scrollView = widget.newScrollView({
		width = screenWidth - 100,
		height = screenHeight - 100,
		topPadding = 15,
		bottomPadding = 15,
		leftPadding = 15,
		rightPadding = 15,
		x = centerX,
		y = screenBottom + screenHeight,
		horizontalScrollDisabled = true,
	})
	scrollView:addEventListener("tap", function() return true end)
	frontGroup:insert(1, scrollView)

	local textObjects = {}

	for i = 1, #results do
		local text = display.newText({
			text = results[i],
			width = scrollView.width - 60,
			font = "courier",
			fontSize = 16,
		})
		text.anchorX, text.anchorY = 0, 0
		text.x = 30
		if textObjects[i-1] then
			text.y = textObjects[i-1].y + textObjects[i-1].height
		else
			text.y = 0
		end
		scrollView:insert(text)
		text:setFillColor(0)
		if isError then text:setFillColor(1, 0, 0) end
		textObjects[i] = text
	end

	local blackout = display.newRect(frontGroup, centerX, centerY, screenWidth, screenHeight)
	blackout:setFillColor(0, 0, 0, .6)
	blackout.alpha = 0
	blackout.isHitTestable = true

	local function closeResults()

		local function remove()
			display.remove(blackout)
			display.remove(scrollView)
			showButtons()
		end

		local function part4()
			scrollView:toBack()
			transition.to(scrollView, {y = screenBottom + screenHeight, time = 800, transition = easing.inOutSine, onComplete = remove})

			transition.to(failwhale, {y = failwhale.targetY, time = 1000, transition = easing.inOutSine, onComplete = whaleMotion})
		end

		local function part3()
			transition.to(blackout, {alpha = 0})
			transition.to(scrollView, {y = screenTop + screenHeight*.25, time = 300, transition = easing.inOutSine, onComplete = part4})
		end

		if blackout.alpha == 1 then
			blackout.tap = function() return true end
			part3()
		end
		return true
	end
	blackout.tap = closeResults
	blackout:addEventListener("tap")
	blackout:addEventListener("touch", function() return true end)

	local function part2()
		scrollView:toFront()
		transition.to(scrollView, {y = centerY, time = 300, transition = easing.inOutSine, onComplete = part3})
		transition.to(blackout, {alpha = 1})
	end

	transition.to(scrollView, {y = screenTop + screenHeight*.25, time = 800, transition = easing.inOutSine, onComplete = part2})
end

--------------------------------------------------------------------------------
-- ADD MOTION TO WAVES & WHALE
--------------------------------------------------------------------------------
function waveMotion(wave)
	local targetX = math.random(30, 60)
	local targetY = math.random(2, 8)
	if wave.x >= 0 then targetX = -targetX end
	if wave.y > 0 then targetY = 0 end

	transition.to(wave, {x = targetX, y = targetY, time = math.random(4000, 8000), transition = easing.inOutSine, onComplete = waveMotion})
end

function whaleMotion(direction)
	local function vertical()
		local random = math.random(10, 50)
		if failwhale.y >= failwhale.targetY then random = -random end
		local target = failwhale.targetY + random
		failwhale.vert = transition.to(failwhale, {y = target, time = math.random(3000, 5000), transition = easing.inOutSine, onComplete = vertical})
	end

	local function horizontal()
		local random = math.random(100, 250)
		local xScale = -1
		local halfway = -.01
		if failwhale.x >= centerX then
			random = -random
			xScale = 1
			halfway = .01
		end
		local target = centerX + random
		transition.to(failwhale, {xScale = halfway, transition = easing.inSine, time = 100, onComplete = function()
			failwhale.xScale = -halfway
			transition.to(failwhale, {xScale = xScale, transition = easing.outSine, time = 100})
		end})
		failwhale.horiz = transition.to(failwhale, {x = target, time = math.random(6000, 8000), transition = easing.inOutSine, onComplete = horizontal})
	end

	if direction == "vertical" then
		vertical()
	elseif direction == "horizontal" then
		horizontal()
	else
		vertical()
		horizontal()
	end
end

--------------------------------------------------------------------------------
-- LOG IN:
--------------------------------------------------------------------------------
function login()
	if not loggedIn then
		local function callback(response)
			loggedIn = true
			showButtons()
			transition.cancel(loginText)
			transition.cancel(failwhale.vert)
			transition.to(loginText, {y = 0, alpha = 0})
			failwhale.targetY = centerY - screenHeight * .3
			transition.to(failwhale, {y = failwhale.targetY, time = 600, transition = easing.inOutSine, onComplete = function()whaleMotion("vertical") end})

			userText = display.newText({
				parent = waves,
				text = "logged in as @"..(response.screenName or ""),
				font = "dosis",
				fontSize = 20,
			})
			userText.anchorX, userText.anchorY = 1, 1
			userText.x, userText.y = screenWidth*.5 - 20, -20

			local function imageListener(event)
				local function logoutAsk()
					local function listener(event)
						if event.action == "clicked" and event.index == 2 then
							logout()
						end
					end
					native.showAlert( "Log out?", "Are you sure you want to log out as @"..response.screenName.."?", { "No", "Yes" }, listener )
				end

				local image = event.target
				if image.width > 125 then
					image.xScale = 125/image.width
					image.yScale = image.xScale
				end
				waves:insert(image)
				image.anchorX, image.anchorY = 1, 1
				image.x, image.y = userText.x, userText.y - userText.height - 10
				image:addEventListener("tap", logoutAsk)
				userImage = image
			end

			twitter.getProfileImage( response.screenName, imageListener)
		end

		local function getImage()
			twitter.getUser(callback)
		end

		if twitter.user.screenName then
			callback(twitter.user)
		else
			twitter.login(function() twitter.getUser(callback) end)
		end
	end
end


--------------------------------------------------------------------------------
-- LOG OUT:
--------------------------------------------------------------------------------
function logout()
	if loggedIn then
		hideButtons()
		twitter.logout()
		loggedIn = false
		display.remove(userText)
		display.remove(userImage)
		friends = {}

		transition.cancel(failwhale.vert)
		failwhale.targetY = screenBottom - 512
		transition.to(failwhale, {y = failwhale.targetY, time = 600, transition = easing.inOutSine, onComplete = function()whaleMotion("vertical") end})
		transition.cancel(loginText)
		transition.to(loginText, {alpha = .45, y = -275, time = 1200})
	end
end

--------------------------------------------------------------------------------
-- GET FOLLOWERS
--------------------------------------------------------------------------------
function getFollowers()
	local function go()
		twitter.getFollowers(showResults)
	end

	if loggedIn then
		native.showAlert("twitter.getFollowers()", "twitter.getFollowers() passes a table containing a Twitter user's followers into a callback function. Tap OK to see the contents of this table for @" .. twitter.user.screenName, {"OK"}, go)
	end
end

--------------------------------------------------------------------------------
-- GET FRIENDS
--------------------------------------------------------------------------------
function getFriends()
	local function go()
		local function callback(results, isError, event)
			friends = results
			showResults(results, isError, event)
		end
		twitter.getFriends(callback)
	end

	if loggedIn then
		native.showAlert("twitter.getFriends()", "twitter.getFriends() passes a table containing a Twitter user's friends (the people he or she follows) into a callback function. Tap OK to see the contents of this table for @" .. twitter.user.screenName, {"OK"}, go)
	end
end

--------------------------------------------------------------------------------
-- POST TWEET
--------------------------------------------------------------------------------
function tweet()
	local function go(event)
		local function showTweet(response, isError, event)
			if isError then
				showResults(response, isError, event)
				return true
			end

			local webview, blackout, url

			local function close()
				blackout.tap = function() return true end
				display.remove(webview)
				transition.to(blackout, {alpha = 0, onComplete = function() display.remove(blackout) end})
			end

			blackout = display.newRect(frontGroup, centerX, centerY, screenWidth, screenHeight)
			blackout:setFillColor(0, 0, 0, .6)
			blackout.alpha = 0
			blackout.isHitTestable = true
			blackout.tap = close
			blackout:addEventListener("tap")
			blackout:addEventListener("touch", function() return true end)

			url = "http://www.twitter.com/"..twitter.user.screenName.."/statuses/"..response.id
			webview = native.newWebView(centerX, centerY, screenWidth, screenHeight*.6)
			webview:request(url)
			webview:addEventListener("urlRequest", function() end)

			transition.to(blackout, {alpha = 1})
		end

		if event.action == "clicked" and event.index == 2 then
			twitter.tweet("Tweeted from @schroederapps #CoronaSDK Twitter plugin. Now supporting image uploads! http://twitter.plugin.jasonschroeder.com", "yes.jpg", showTweet)
		end
	end

	if loggedIn then
		native.showAlert("twitter.tweet()", [[twitter.tweet() posts a status update and optional image to the logged-in user's timeline. Tap OK to tweet "Tweeted from the @schroederapps #CoronaSDK Twitter plugin. Now supporting image uploads!" to @]] .. twitter.user.screenName..[['s timeline.]], {"Cancel", "OK"}, go)
	end
end

--------------------------------------------------------------------------------
-- SEARCH TWITTER
--------------------------------------------------------------------------------
function search()
	local function go()
		twitter.search("#CoronaSDK", showResults, {retweets = false})
	end

	if loggedIn then
		native.showAlert("twitter.search()", [[twitter.search() searches Twitter for a term you specify and passes a table with the results into a callback function. Tap okay to search Twitter for "#CoronaSDK" and see the table it returns.]], {"OK"}, go)
	end
end

--------------------------------------------------------------------------------
-- GET USER INFO
--------------------------------------------------------------------------------
function getUser()
	local names = {}
	local function go(event)
		if event.action == "clicked" then
			local name = names[event.index]
			twitter.getUser(name, showResults)
		end
	end

	if loggedIn then
		local function chooseUser(response)
			friends = response
			names = {twitter.user.screenName}
			local displayNames = {"@"..twitter.user.screenName}
			for i = 1, 4 do
				if #response < i then break end
				local num = math.random(#response)
				names[#names+1] = response[num].screenName
				displayNames[#displayNames+1] = "@"..response[num].screenName
				table.remove(response, num)
			end
			native.showAlert("twitter.getUser()", "twitter.getUser() passes a table containing details on a Twitter account into a callback function. Choose a screen name below to see the table returned for that account.", displayNames, go)
		end

		if #friends == 0 then
			twitter.getFriends(chooseUser)
		else
			chooseUser(friends)
		end
	end
end


--------------------------------------------------------------------------------
-- GET PROFILE IMAGE
--------------------------------------------------------------------------------
function getProfileImage()
	local names = {}

	local function showImage(event)
		local image = event.target
		image.x, image.y = centerX, screenBottom + screenHeight
		frontGroup:insert(1, image)

		transition.cancel(failwhale)
		transition.to(failwhale, {y = screenBottom + centerY, time = 800, transition = easing.inOutBack})

		if image.width > screenWidth * .9 then
			image.targetScale = screenWidth * .9 / image.width
		else
			image.targetScale = 1
		end
		image.xScale, image.yScale = image.targetScale * .01, image.targetScale * .01


		local blackout = display.newRect(frontGroup, centerX, centerY, screenWidth, screenHeight)
		blackout:setFillColor(0, 0, 0, .6)
		blackout.alpha = 0
		blackout.isHitTestable = true

		local function closeResults()

			local function remove()
				display.remove(blackout)
				display.remove(scrollView)
				showButtons()
			end

			local function part4()
				image:toBack()
				transition.to(image, {y = screenBottom + screenHeight, xScale = image.targetScale * .01, yScale = image.targetScale * .01, time = 800, transition = easing.inOutSine, onComplete = remove})

				transition.to(failwhale, {y = failwhale.targetY, time = 1000, transition = easing.inOutSine, onComplete = whaleMotion})
			end

			local function part3()
				transition.to(blackout, {alpha = 0})
				transition.to(image, {y = screenTop + screenHeight*.25, time = 300, transition = easing.inOutSine, onComplete = part4})
			end

			if blackout.alpha == 1 then
				blackout.tap = function() return true end
				part3()
			end
			return true
		end

		blackout.tap = closeResults
		blackout:addEventListener("tap")
		blackout:addEventListener("touch", function() return true end)

		local function part2()
			image:toFront()
			transition.to(image, {y = centerY, time = 300, transition = easing.inOutSine})
			transition.to(blackout, {alpha = 1})
		end

		transition.to(image, {y = screenTop + screenHeight*.25, xScale = image.targetScale, yScale = image.targetScale, time = 800, transition = easing.inOutSine, onComplete = part2})
	end

	local function go(event)
		if event.action == "clicked" then
			local name = names[event.index]
			hideButtons()
			twitter.getProfileImage(name, showImage)
		end
	end

	if loggedIn then
		local function chooseUser(response, isError, event)
			if isError then
				showResults(response, isError, event)
				return true
			end
			friends = response
			local displayNames = {}
			for i = 1, 5 do
				if #response < i then break end
				local num = math.random(#response)
				names[#names+1] = response[num].screenName
				displayNames[#displayNames+1] = "@"..response[num].screenName
				table.remove(response, num)
			end
			native.showAlert("twitter.getProfileImage()", "twitter.getProfileImage is a modified version of Corona's display.loadRemoteImage API that returns a display object containing a Twitter user's profile image. Choose a screen name below to load that user's profile image.", displayNames, go)
		end

		if #friends == 0 then
			twitter.getFriends(chooseUser)
		else
			chooseUser(friends)
		end
	end
end


--------------------------------------------------------------------------------
-- FOLLOW JASON
--------------------------------------------------------------------------------
function follow()
	local function thanks(response, isError, event)
		if isError then
			showResults(response, isError, event)
			return true
		end
		if response.following then
		native.showAlert("Thanks!", "@"..twitter.user.screenName.." is now following @schroederapps on Twitter. Thanks for using the plugin. Enjoy! \n \n -Jason Schroeder \n @schroederapps", {"You're welcome!"})
		end
	end

	local function go(event)
		if event.action == "clicked" and event.index == 2 then
			twitter.follow("schroederapps", thanks)
		end
	end

	native.showAlert("twitter.follow()", "twitter.follow() makes the user a follower of a Twitter account you specify. It passes a table containing info on that account into a callback function. Tap OK to use this API to follow @schroederapps.", {"Cancel", "OK"}, go)
end


--------------------------------------------------------------------------------
-- FLEXIBLE REQUEST
--------------------------------------------------------------------------------
function flexibleRequest()

	showResults([[*************************************************************
****************** ABOUT TWITTER.REQUEST() ******************
*************************************************************

twitter.request() is the backbone of the Twitter plugin. It allows you to easily make an API call against any of Twitter's public REST API endpoints (https://dev.twitter.com/rest/public), and pass the resulting JSON data into a callback function. All the "convenience functions" built into the plugin like twitter.tweet() and twitter.search() use twitter.request() behind the scenes, and then they convert the returned JSON data to a more manageable Lua table. You can easily recreate those functions, then tweak them to your liking, by using twitter.request().

twitter.request() requires a single Lua table as an argument, containing the following key/value pairs:

*** endpoint : a string representing the API endpoint you want to use (e.g. "statuses/retweets_of_me" or "direct_messages")

*** method : the HTTP method employed by the endpoint you are using. As of July 2015, the only methods used are "GET" and "POST" (and that's unlikely to change).

*** parameters : a Lua table containing key/value pairs representing the required and/or optional parameters for the endpoint you are using. Check Twitter's API docs for details.

*** callback : a Lua function that accepts up to three arguments: 1.) the returned JSON data string (required); 2.) a boolean indicating if there was an error (optional); 3.) a Corona network event table in the event of an error (optional).

Below is an simple twitter.request() call that would search Twitter for the phrase "@schroederapps" and pass the resulting JSON string into a function called "myCallback". Note that there are optional parameters that are not being used in this example, but they could be by adding key/value pairs to the "parameters" table:

twitter.request({
	endpoint = "search/tweets",
	method = "GET",
	parameters = {
		q = "@schroederapps",
	},
	callback = myCallback,
})
]], nil, nil, true)
end
--------------------------------------------------------------------------------
-- SHOW BUTTONS
--------------------------------------------------------------------------------
function showButtons()
	if loggedIn then
		local left = screenLeft + 25
		local right = screenRight - 25
		transition.to(tweetButton, {x = left, transition = easing.inOutBack, delay = 300})
		transition.to(searchButton, {x = left, transition = easing.inOutBack, delay = 200})
		transition.to(followButton, {x = left, transition = easing.inOutBack, delay = 100})
		transition.to(flexibleRequestButton, {x = left, transition = easing.inOutBack, delay = 0})
		transition.to(getUserButton, {x = right, transition = easing.inOutBack, delay = 400})
		transition.to(getFollowersButton, {x = right, transition = easing.inOutBack, delay = 500})
		transition.to(getFriendsButton, {x = right, transition = easing.inOutBack, delay = 600})
		transition.to(getProfileImageButton, {x = right, transition = easing.inOutBack, delay = 700})
	end
end

--------------------------------------------------------------------------------
-- HIDE BUTTONS
--------------------------------------------------------------------------------
function hideButtons()
	local left = screenLeft - centerX
	local right = screenRight + centerX
	transition.to(tweetButton, {x = left, time = 300})
	transition.to(searchButton, {x = left, time = 300})
	transition.to(followButton, {x = left, time = 300})
	transition.to(flexibleRequestButton, {x = left, time = 300})
	transition.to(getUserButton, {x = right, time = 300})
	transition.to(getFollowersButton, {x = right, time = 300})
	transition.to(getFriendsButton, {x = right, time = 300})
	transition.to(getProfileImageButton, {x = right, time = 300})
end

--------------------------------------------------------------------------------
-- CREATE VISUALS & UI
--------------------------------------------------------------------------------
backGroup = display.newGroup()
frontGroup = display.newGroup()

bg = display.newRect(backGroup, centerX, centerY, screenWidth, screenHeight)
bg:setFillColor(108/255, 197/255, 195/255)

failwhale = display.newImage(backGroup, "failwhale.png")
failwhale.x, failwhale.y = centerX + 20, centerY + screenHeight
failwhale.xScale = -1
failwhale.targetY = screenBottom - 512
failwhale:addEventListener("tap", login)

waves = display.newGroup()
waves.x, waves.y = centerX, screenBottom + 300

loginText = display.newText({
	parent = waves,
	text = "tap fail whale to log in",
	font = "dosis",
	fontSize = 45,
})
loginText.anchorY = 1
loginText.x, loginText.y = 0, 0
loginText.alpha = 0

wave1 = display.newImage(waves, "waves.png", true)
wave1.anchorY = 1

wave2 = display.newImage(waves, "waves.png", true)
wave2:setFillColor(251/255, 90/255, 34/255)
wave2.anchorY = 1

waveMotion(wave1)
waveMotion(wave2)

frontGroup:insert(waves)

subtitle = display.newText({
	parent = waves,
	text = "for Corona SDK by @schroederapps",
	font = "dosis",
	fontSize = 30,
})
subtitle.anchorX, subtitle.anchorY = 0, 1
subtitle.x, subtitle.y = -screenWidth*.5 + 20, -20

title = display.newText({
	parent = waves,
	text = "Twitter plugin",
	font = "dosis",
	fontSize = 60,
})
title.anchorX, title.anchorY = 0, 1
title.x, title.y = subtitle.x, subtitle.y - subtitle.height

tweetButton = widget.newButton({
	label = "twitter.tweet()",
    onRelease = tweet,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(tweetButton)
tweetButton:addEventListener("tap", function() return true end)
tweetButton.anchorX, tweetButton.anchorY = 0, 1
tweetButton.x, tweetButton.y = screenLeft - centerX, screenBottom - 550

searchButton = widget.newButton({
	label = "twitter.search()",
    onRelease = search,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(searchButton)
searchButton:addEventListener("tap", function() return true end)
searchButton.anchorX, searchButton.anchorY = 0, 1
searchButton.x, searchButton.y = tweetButton.x, tweetButton.y + 80

followButton = widget.newButton({
	label = "twitter.follow()",
    onRelease = follow,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(followButton)
followButton:addEventListener("tap", function() return true end)
followButton.anchorX, followButton.anchorY = 0, 1
followButton.x, followButton.y = searchButton.x, searchButton.y + 80

flexibleRequestButton = widget.newButton({
	label = "twitter.request()",
    onRelease = flexibleRequest,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={6/255, 28/255, 53/255, 1}, over={6/255, 28/255, 53/255, .4}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(flexibleRequestButton)
flexibleRequestButton:addEventListener("tap", function() return true end)
flexibleRequestButton.anchorX, flexibleRequestButton.anchorY = 0, 1
flexibleRequestButton.x, flexibleRequestButton.y = followButton.x, followButton.y + 80

getUserButton = widget.newButton({
	label = "twitter.getUser()",
    onRelease = getUser,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(getUserButton)
getUserButton:addEventListener("tap", function() return true end)
getUserButton.anchorX, getUserButton.anchorY = 1, 1
getUserButton.x, getUserButton.y = screenRight + centerX, tweetButton.y

getFollowersButton = widget.newButton({
	label = "twitter.getFollowers()",
    onRelease = getFollowers,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(getFollowersButton)
getFollowersButton:addEventListener("tap", function() return true end)
getFollowersButton.anchorX, getFollowersButton.anchorY = 1, 1
getFollowersButton.x, getFollowersButton.y = getUserButton.x, getUserButton.y + 80

getFriendsButton = widget.newButton({
	label = "twitter.getFriends()",
    onRelease = getFriends,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(getFriendsButton)
getFriendsButton:addEventListener("tap", function() return true end)
getFriendsButton.anchorX, getFriendsButton.anchorY = 1, 1
getFriendsButton.x, getFriendsButton.y = getFollowersButton.x, getFollowersButton.y + 80

getProfileImageButton = widget.newButton({
	label = "twitter.getProfileImage()",
    onRelease = getProfileImage,
    font = "courier",
    fontSize = 22,
    shape="roundedRect",
    width = 345,
    height = 60,
    cornerRadius = 20,
    fillColor = {default={251/255, 90/255, 34/255, 1}, over={251/255, 90/255, 34/255, .6}},
    labelColor = { default={ 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeColor = { default={ 1, 1, 1, 1 }, over={ 1, 1, 1, 1 } },
    strokeWidth = 4
})
backGroup:insert(getProfileImageButton)
getProfileImageButton:addEventListener("tap", function() return true end)
getProfileImageButton.anchorX, getProfileImageButton.anchorY = 1, 1
getProfileImageButton.x, getProfileImageButton.y = getFriendsButton.x, getFriendsButton.y + 80

--------------------------------------------------------------------------------
-- TRANSITION IN ON APP LAUNCH
--------------------------------------------------------------------------------
transition.to(waves, {y = screenBottom, time = 1000, transition = easing.inOutSine})
failwhale.vert = transition.to(failwhale, {y = failwhale.targetY, time = 1800, transition = easing.inOutSine, onComplete = whaleMotion})
transition.to(loginText, {alpha = .45, y = -275, time = 1000, delay = 1800})

--------------------------------------------------------------------------------
-- CHECK FOR SAVED TWITTER USER DATA
--------------------------------------------------------------------------------
if twitter.user.screenName then
	transition.cancel(loginText)
	transition.cancel(failwhale.vert)
	failwhale.targetY = centerY - screenHeight*.3
	failwhale.vert = transition.to(failwhale, {y = failwhale.targetY, time = 1800, transition = easing.inOutSine, onComplete = function() whaleMotion(); login() end})
end

if twitter.apiKey == "API_KEY" or twitter.apiSecret == "API_SECRET" then
	native.showAlert("Twitter plugin not initialized", "You need to register a Twitter app and add your app's API Key and API Secret to the twitter.init() function call on line 15 of main.lua for this demo to work. See the plugin docs at jasonschroeder.com for more details.", {"OK"})
end
